#include <stdbool.h>
#include "nrf.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_saadc.h"
#include "nrf_saadc.h"
#include "nrf_gpiote.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_drv_power.h"
#include "nrf_delay.h"

#include "boards.h"
#include "app_error.h"


// P0.04
#define X_PLUS             4
#define X_PLUS_ADC         NRF_SAADC_INPUT_AIN2
#define X_PLUS_ADC_CHANNEL 0

// P0.05
#define Y_PLUS             5
#define Y_PLUS_ADC         NRF_SAADC_INPUT_AIN3
#define Y_PLUS_ADC_CHANNEL 1

// P0.06
#define X_MINUS            6

// P0.07
#define Y_MINUS            7

#define DEBOUNCE_MS        10
#define SETTLING_US        20

// number of A/D samples per axis
#define NUM_SAMPLES        16 

// array to store x_samples
nrf_saadc_value_t x_values[NUM_SAMPLES]; 

// array to store y_samples
nrf_saadc_value_t y_values[NUM_SAMPLES]; 


// variable to store x position
nrf_saadc_value_t x_position; 

// variable to store y position
nrf_saadc_value_t y_position; 

// Function Prototype's
void config_pins_x (void);
void config_pins_y (void);
void config_pins_touch (void);
bool touch_detect (void);
void read_ch_x (void);
void read_ch_y (void);


// Event handler function prototype's
void xplus_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action);
void saadc_callback(nrf_drv_saadc_evt_t const * p_event);

/**
 * @brief Function for application main entry.
 */
int main(void)
{
  // return code	
  ret_code_t err_code;	

  // Initialize power module driver
  err_code = nrf_drv_power_init(NULL);
  APP_ERROR_CHECK(err_code);

  // Initialize power management driver
  err_code = nrf_pwr_mgmt_init(0);
  APP_ERROR_CHECK(err_code);
  
  // Initialize GPIOTE driver
  err_code = nrf_drv_gpiote_init();
  APP_ERROR_CHECK(err_code);  
  
  // Initialize SAADC driver
  err_code = nrf_drv_saadc_init(NULL, saadc_callback);
  APP_ERROR_CHECK(err_code);  
  
  // Initialize touch detect
  touch_detect();
  
  // Initialize x plus pin to detect low edge
  nrf_drv_gpiote_in_config_t xplus_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
  xplus_config.pull = NRF_GPIO_PIN_PULLUP; 
  err_code = nrf_drv_gpiote_in_init(X_PLUS, &xplus_config, xplus_pin_handler);
  APP_ERROR_CHECK(err_code);
  
  // enable the event for x plus pin
  nrf_drv_gpiote_in_event_enable(X_PLUS, true);
 
  // loop forever
  while (true)
  {	 
     // wait for an event
	 nrf_pwr_mgmt_run();   
  } 
  
}

/**
 * @brief Function to detect touch.
 */
bool touch_detect (void)
{
  // configure the pins for touch detect
  config_pins_touch();
  
  // settling time for switching
  nrf_delay_us(SETTLING_US);  
  
  // return TRUE if x plus pin is low
  return (!nrf_drv_gpiote_in_is_set(X_PLUS));  
}


/**
 * @brief Function to configure pins to measure x position.
 */
void config_pins_x (void)
{
  // return code	
  ret_code_t err_code;	
	
  // Configure X MINUS as a digital Output with no pullup and set it LOW
  nrf_gpio_cfg_output(X_MINUS);
  nrf_gpio_pin_clear(X_MINUS);
  
  // Configure Y MINUS as a digital Input with no pullup  
  nrf_gpio_cfg_input(Y_MINUS, NRF_GPIO_PIN_NOPULL);
  
  // Configure Y PLUS as an analog Input (SAADC pin)
  nrf_saadc_channel_config_t yplus_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(Y_PLUS_ADC);
		
  err_code = nrf_drv_saadc_channel_init(Y_PLUS_ADC_CHANNEL, &yplus_config);
  APP_ERROR_CHECK(err_code);		
  
  // Configure X PLUS as a digital Output with no pullup and set it HIGH
  nrf_gpio_cfg_output(X_PLUS);
  nrf_gpio_pin_set(X_PLUS);
  
}


/**
 * @brief Function to configure pins to measure y position.
 */
void config_pins_y (void)
{
  // return code
  ret_code_t err_code;		
	
  // Configure X MINUS as a digital Input with no pullup 
  nrf_gpio_cfg_input(X_MINUS, NRF_GPIO_PIN_NOPULL); 
  
  // Configure Y MINUS as a digital Output with no pullup and set it LOW
  nrf_gpio_cfg_output(Y_MINUS);
  nrf_gpio_pin_clear(Y_MINUS);
    
  // Configure Y PLUS as a digital Output with no pullup and set it HIGH
  nrf_gpio_cfg_output(Y_PLUS);
  nrf_gpio_pin_set(Y_PLUS);
  
  // Configure X PLUS as an analog Input (SAADC pin)
  nrf_saadc_channel_config_t xplus_config =
        NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(X_PLUS_ADC);
		
  err_code = nrf_drv_saadc_channel_init(X_PLUS_ADC_CHANNEL, &xplus_config);
  APP_ERROR_CHECK(err_code);		
}

/**
 * @brief Function to configure pins to detect touch on x plus pin.
 */
void config_pins_touch (void)
{ 
	
  // Configure X MINUS as a digital Input with no pullup   
  nrf_gpio_cfg_input(X_MINUS, NRF_GPIO_PIN_NOPULL); 
  
  // Configure Y MINUS as a digital Output with no pullup and set it LOW
  nrf_gpio_cfg_output(Y_MINUS);
  nrf_gpio_pin_clear(Y_MINUS);
  
  // Configure Y PLUS as a digital Input with no pullup  
  nrf_gpio_cfg_input(Y_PLUS, NRF_GPIO_PIN_NOPULL);
  
  // Configure X PLUS as a digital Input with pullup  
  nrf_gpio_cfg_input(X_PLUS, NRF_GPIO_PIN_PULLUP); 
      
}

/**
 * @brief Function to measure x position.
 */
void read_ch_x (void)
{  	
  // configure pins for read x-dir
  config_pins_x(); 
  
  // settling time for switching  
  nrf_delay_us(SETTLING_US);
 
  // sample the x plus pin to get x-dir
  for(uint8_t i = 0; i < NUM_SAMPLES; ++i) 
	nrf_drv_saadc_sample_convert(Y_PLUS_ADC_CHANNEL, &x_values[i]); 
  
}

/**
 * @brief Function to measure y position.
 */
void read_ch_y (void)
{
  // configure pins for read y-dir
  config_pins_y(); 
  
  // settling time for switching  
  nrf_delay_us(SETTLING_US);
  
  // sample the x plus pin to get y-dir
  for(uint8_t i = 0; i < NUM_SAMPLES; ++i) 
	nrf_drv_saadc_sample_convert(X_PLUS_ADC_CHANNEL, &y_values[i]);    
}

/**
 * @brief X plus pin low edge detect event handler 
 */
void xplus_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
  // return code
  ret_code_t err_code;			
 
  // disable the GPIO event on x plus pin  
  nrf_drv_gpiote_in_event_disable(X_PLUS);
 
  // debounce the touch
  nrf_delay_ms(DEBOUNCE_MS);  
 
  // loop as long as screen is touched
  while(touch_detect())
  {
	// read and collect the x values  
	read_ch_x(); 
	
	// read and collect the y values	 	 	 
    read_ch_y();

	// initial values
    x_position = 0;	
	y_position = 0;	
	
	// add up the x conversion results
	for(uint8_t i = 0; i < NUM_SAMPLES; ++i) 
	  x_position += x_values[i];
	
	// add up the y conversion results
    for(uint8_t i = 0; i < NUM_SAMPLES; ++i) 
	  y_position += y_values[i];
	
	// get average
	x_position = (x_position/NUM_SAMPLES);
	y_position = (y_position/NUM_SAMPLES);
	
  } 
  
  // enable the event for x plus pin
  nrf_drv_gpiote_in_event_enable(X_PLUS, true);
 
}

/**
 * @brief SAADC event handler
 */
void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{	
}

